@@ -71,6 +71,10 @@ public final class Buffer implements BufferedSource, BufferedSink, Cloneable, By
     return this;
   }
 
+  @Override public Buffer getBuffer() {
+    return this;
+  }
+
   @Override public OutputStream outputStream() {
     return new OutputStream() {
       @Override public void write(int b) {
@@ -113,6 +117,10 @@ public final class Buffer implements BufferedSource, BufferedSink, Cloneable, By
     return size >= byteCount;
   }
 
+  @Override public BufferedSource peek() {
+    return Okio.buffer(new PeekSource(this));
+  }
+
   @Override public InputStream inputStream() {
     return new InputStream() {
       @Override public int read() {
