@@ -27,9 +27,17 @@ import javax.annotation.Nullable;
  * input.
  */
 public interface BufferedSource extends Source, ReadableByteChannel {
-  /** Returns this source's internal buffer. */
+  /**
+   * Returns this source's internal buffer.
+   *
+   * @deprecated use getBuffer() instead.
+   */
+  @Deprecated
   Buffer buffer();
 
+  /** This source's internal buffer. */
+  Buffer getBuffer();
+
   /**
    * Returns true if there are no more bytes in this source. This will block until there are bytes
    * to read or the source is definitely exhausted.
@@ -533,6 +541,29 @@ public interface BufferedSource extends Source, ReadableByteChannel {
   boolean rangeEquals(long offset, ByteString bytes, int bytesOffset, int byteCount)
       throws IOException;
 
+  /**
+   * Returns a new {@code BufferedSource} that can read data from this {@code BufferedSource}
+   * without consuming it. The returned source becomes invalid once this source is next read or
+   * closed.
+   *
+   * For example, we can use {@code peek()} to lookahead and read the same data multiple times.
+   *
+   * <pre> {@code
+   *
+   *   Buffer buffer = new Buffer();
+   *   buffer.writeUtf8("abcdefghi");
+   *
+   *   buffer.readUtf8(3) // returns "abc", buffer contains "defghi"
+   *
+   *   BufferedSource peek = buffer.peek();
+   *   peek.readUtf8(3); // returns "def", buffer contains "defghi"
+   *   peek.readUtf8(3); // returns "ghi", buffer contains "defghi"
+   *
+   *   buffer.readUtf8(3); // returns "def", buffer contains "ghi"
+   * }</pre>
+   */
+  BufferedSource peek();
+
   /** Returns an input stream that reads from this source. */
   InputStream inputStream();
 }
